/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.communication.jndi.agent;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.naming.ServiceUnavailableException;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.communication.jndi.agent.util.RegistryUtil;
import org.ow2.petals.kernel.configuration.ConfigurationService;
import org.ow2.petals.util.LoggingUtil;

/**
 * The standalone implementation of the PEtALS JNDI agent
 * {@link JNDIAgentService}.
 * 
 * @author rnaudin - EbmWebsourcing
 * @since Petals 2.3
 * 
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.communication.jndi.agent.JNDIAgentService.class))
public class StandaloneJNDIAgentServiceImpl extends AbstractJNDIAgentServiceImpl {

    /**
     * Logging utilities
     */

    @Monolog(name = "logger")
    private Logger logger;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration", signature = org.ow2.petals.kernel.configuration.ConfigurationService.class)
    private ConfigurationService configurationService;

    @Override
    public boolean containsContext(String key) throws ServiceUnavailableException {
        return this.data.containsKey(key);
    }

    @Override
    public Map<String, Serializable> getContext(String key) throws ServiceUnavailableException {
        this.log.start(key);

        Map<String, Serializable> contextMap = this.data.get(key);
        if (contextMap == null) {
            contextMap = new Hashtable<String, Serializable>();
            this.data.put(key, contextMap);
        }

        this.log.end();
        return contextMap;
    }

    @Override
    public void removeContext(String key) throws ServiceUnavailableException {
        this.log.call(key);

        this.data.remove(key);
    }

    /**
     * Initialize the JNDI data
     */
    private void initData() {
        this.log.start();

        this.data = new Hashtable<String, Map<String, Serializable>>();
        this.data.put("/", new HashMap<String, Serializable>());

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(this.logger);
        this.log.start();

        this.domainConfiguration = this.configurationService.getDomainConfiguration();

        if (this.domainConfiguration.getJndiConfiguration() == null) {
            this.initData();
            this.registryUtil = new RegistryUtil(this);
        }

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.call();
    }

}
