/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: InstallationTaskFactory.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.installation;

import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.container.ContainerService;
import org.ow2.petals.jbi.management.admin.AdminService;
import org.ow2.petals.jbi.management.task.CopyArchiveToInstalledDirTask;
import org.ow2.petals.jbi.management.task.CopyArchiveToWorkDirTask;
import org.ow2.petals.jbi.management.task.DownloadTask;
import org.ow2.petals.jbi.management.task.ExtractTask;
import org.ow2.petals.jbi.management.task.installation.install.component.ComponentCheckPackageTask;
import org.ow2.petals.jbi.management.task.installation.install.component.CreateAndRegisterInstallerMBeanTask;
import org.ow2.petals.jbi.management.task.installation.install.component.CreateComponentRepositoryTask;
import org.ow2.petals.jbi.management.task.installation.install.component.CreateComponentStateHolderTask;
import org.ow2.petals.jbi.management.task.installation.install.component.RegisterComponentInSLTask;
import org.ow2.petals.jbi.management.task.installation.install.sl.CreateSLClassloaderTask;
import org.ow2.petals.jbi.management.task.installation.install.sl.CreateSLRepositoryTask;
import org.ow2.petals.jbi.management.task.installation.install.sl.CreateSLStateHolderTask;
import org.ow2.petals.jbi.management.task.installation.install.sl.SLCheckPackageTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.component.RemoveComponentArchiveTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.component.RemoveComponentRepositoryTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.component.RemoveComponentStateHolderTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.component.UnregisterComponentFromSLTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.component.UnregisterInstallerTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.sl.RemoveSLArchiveTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.sl.RemoveSLClassloaderTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.sl.RemoveSLRepositoryTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.sl.RemoveSLStateHolderTask;
import org.ow2.petals.jbi.messaging.routing.RouterService;
import org.ow2.petals.system.repository.RepositoryService;
import org.ow2.petals.system.systemstate.SystemStateService;
import org.ow2.petals.util.LoggingUtil;

/**
 * This Factory creates Installation Tasks.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class InstallationTaskFactory {
    private final CopyArchiveToWorkDirTask copyArchiveToWorkDirTask;

    private final RegisterComponentInSLTask registerComponentInSLTask;

    private final CreateComponentRepositoryTask createComponentFolderTask;

    private final ComponentCheckPackageTask componentCheckPackageTask;

    private final CreateComponentStateHolderTask createComponentStateHolderTask;

    private final CreateAndRegisterInstallerMBeanTask createAndRegisterInstallerMBeanTask;

    private final CreateSLClassloaderTask createSLClassloaderTask;

    private final CreateSLRepositoryTask createSLRepositoryTask;

    private final SLCheckPackageTask slCheckPackageTask;

    private final CreateSLStateHolderTask createSLStateHolderTask;

    private final CopyArchiveToInstalledDirTask copyArchiveToInstalledDirTask;

    private final RemoveComponentRepositoryTask removeComponentFolderTask;

    private final RemoveComponentStateHolderTask removeComponentStateHolderTask;

    private final RemoveComponentArchiveTask removeComponentArchiveTask;

    private final UnregisterInstallerTask unregisterInstallerTask;

    private final UnregisterComponentFromSLTask unregisterComponentFromSLTask;

    private final RemoveSLArchiveTask removeSLArchiveTask;

    private final RemoveSLClassloaderTask removeSLClassloaderTask;

    private final RemoveSLRepositoryTask removeSLFolderTask;

    private final RemoveSLStateHolderTask removeSLStateHolderTask;

    private final ExtractTask extractTask;

    private final DownloadTask downloadTask;

    /**
     * Creates a new instance of {@link InstallationTaskFactory}
     * 
     * @param packageHandler
     * @param systemStateService
     * @param repositoryService
     * @param log
     * @param logger
     * @param adminService
     * @param router
     * @param loaderService
     */
    public InstallationTaskFactory(SystemStateService systemStateService,
            RepositoryService repositoryService, LoggingUtil log, Logger logger,
            AdminService adminService, RouterService router, ContainerService container,
            String petalsInstallDirectory) {
        super();

        /*
         * Create Tasks
         */
        // Component creation tasks
        this.registerComponentInSLTask = new RegisterComponentInSLTask(adminService);

        this.createComponentFolderTask = new CreateComponentRepositoryTask(log, repositoryService);

        this.componentCheckPackageTask = new ComponentCheckPackageTask(log, adminService);

        this.createComponentStateHolderTask = new CreateComponentStateHolderTask(log,
                systemStateService, petalsInstallDirectory);

        this.createAndRegisterInstallerMBeanTask = new CreateAndRegisterInstallerMBeanTask(log,
                adminService, container);

        // SL creation tasks
        this.createSLClassloaderTask = new CreateSLClassloaderTask(log, adminService, container);

        this.createSLRepositoryTask = new CreateSLRepositoryTask(log, repositoryService);

        this.slCheckPackageTask = new SLCheckPackageTask(log, adminService);

        this.createSLStateHolderTask = new CreateSLStateHolderTask(log, systemStateService,
                petalsInstallDirectory);

        this.copyArchiveToInstalledDirTask = new CopyArchiveToInstalledDirTask(log,
                petalsInstallDirectory);

        // Component removal tasks
        this.removeComponentFolderTask = new RemoveComponentRepositoryTask(log, repositoryService);

        this.removeComponentStateHolderTask = new RemoveComponentStateHolderTask(log,
                systemStateService);

        this.removeComponentArchiveTask = new RemoveComponentArchiveTask(log,
                petalsInstallDirectory);

        this.unregisterInstallerTask = new UnregisterInstallerTask(log, adminService, container);

        this.unregisterComponentFromSLTask = new UnregisterComponentFromSLTask(log, adminService);

        // SL removal tasks
        this.removeSLArchiveTask = new RemoveSLArchiveTask(log, petalsInstallDirectory);

        this.removeSLClassloaderTask = new RemoveSLClassloaderTask(log, adminService, container);

        this.removeSLFolderTask = new RemoveSLRepositoryTask(log, repositoryService);

        this.removeSLStateHolderTask = new RemoveSLStateHolderTask(log, systemStateService);

        // shared tasks
        this.extractTask = new ExtractTask(log, petalsInstallDirectory);

        this.downloadTask = new DownloadTask(log, petalsInstallDirectory);

        this.copyArchiveToWorkDirTask = new CopyArchiveToWorkDirTask(log, petalsInstallDirectory);
    }

    public DownloadTask getDownloadTask() {
        return this.downloadTask;
    }

    public CopyArchiveToWorkDirTask getCopyArchiveToWorkDirTask() {
        return this.copyArchiveToWorkDirTask;
    }

    public RegisterComponentInSLTask getRegisterComponentInSLTask() {
        return this.registerComponentInSLTask;
    }

    public CreateComponentRepositoryTask getCreateComponentFolderTask() {
        return this.createComponentFolderTask;
    }

    public ComponentCheckPackageTask getComponentCheckPackageTask() {
        return this.componentCheckPackageTask;
    }

    public CreateComponentStateHolderTask getCreateComponentStateHolderTask() {
        return this.createComponentStateHolderTask;
    }

    public CopyArchiveToInstalledDirTask getCopyArchiveToInstalledDirTask() {
        return this.copyArchiveToInstalledDirTask;
    }

    public CreateAndRegisterInstallerMBeanTask getCreateAndRegisterInstallerMBeanTask() {
        return this.createAndRegisterInstallerMBeanTask;
    }

    public CreateSLClassloaderTask getCreateSLClassloaderTask() {
        return this.createSLClassloaderTask;
    }

    public CreateSLRepositoryTask getCreateSLRepositoryTask() {
        return this.createSLRepositoryTask;
    }

    public SLCheckPackageTask getSLCheckPackageTask() {
        return this.slCheckPackageTask;
    }

    public CreateSLStateHolderTask getCreateSLStateHolderTask() {
        return this.createSLStateHolderTask;
    }

    public RemoveComponentArchiveTask getRemoveComponentArchiveTask() {
        return this.removeComponentArchiveTask;
    }

    public RemoveSLArchiveTask getRemoveSLArchiveTask() {
        return this.removeSLArchiveTask;
    }

    public UnregisterInstallerTask getUnregisterInstallerTask() {
        return this.unregisterInstallerTask;
    }

    public UnregisterComponentFromSLTask getUnregisterComponentFromSLTask() {
        return this.unregisterComponentFromSLTask;
    }

    public RemoveSLClassloaderTask getRemoveSLClassloaderTask() {
        return this.removeSLClassloaderTask;
    }

    public RemoveSLRepositoryTask getRemoveSLFolderTask() {
        return this.removeSLFolderTask;
    }

    public RemoveSLStateHolderTask getRemoveSLStateHolderTask() {
        return this.removeSLStateHolderTask;
    }

    public RemoveComponentRepositoryTask getRemoveComponentFolderTask() {
        return this.removeComponentFolderTask;
    }

    public RemoveComponentStateHolderTask getRemoveComponentStateHolderTask() {
        return this.removeComponentStateHolderTask;
    }

    public ExtractTask getExtractTask() {
        return this.extractTask;
    }

}
