/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.installation.uninstall.component;

import java.io.File;

import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.jbi.management.task.AbstractFileManipulationTask;
import org.ow2.petals.systemstate.generated.Component;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task moves a component archive from the installed Petals directory to
 * the uninstalled directory<br/>
 * Inputs : {@link ComponentState}<br />
 * The task has got a robust approach : it tries to do all the subtasks<br />
 * and doesn't have the undo capability.<br />
 * 
 * @author ofabre - EBM Websourcing
 * @author fgardes - EBM Websourcing
 */
public class RemoveComponentArchiveTask extends AbstractFileManipulationTask {

    public RemoveComponentArchiveTask(LoggingUtil log, String petalsInstallDirectory) {
        super(log, petalsInstallDirectory);
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.jbi.management.Task#execute(org.ow2.petals.jbi.management.Context)
     */
    public void execute(Context context) throws Exception {
        Component componentStateHolder = context.getComponentStateHolder();
        if (componentStateHolder != null) {
            final File installedArchive = this.getInstalledArchive(componentStateHolder
                    .getArchiveName());
            if ((installedArchive != null) && installedArchive.exists()) {
                installedArchive.delete();
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.Map)
     */
    public void undo(Context context) throws Exception {
        // TODO : recreate the component archive
    }
}
