/**
 * PETALS: PETALS Services Platform Copyright (C) 2005 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: Transporter.java,v 0.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------
 */

package org.ow2.petals.transport;

import org.ow2.petals.jbi.messaging.exchange.MessageExchange;
import org.ow2.petals.transport.util.TransportSendContext;

/**
 * The <code>Transporter</code> is used to send a <code>MessageExchange</code>
 * from a Petals container to another one. <br>
 * The local <code>Router</code> calls
 * <code>send(exchange,distantContainer)</code> on this <code>Transporter</code>
 * to send an exchange. <br>
 * This <code>Transporter</code> sends the <code>MessageExchange</code> to the
 * distant <code>Transporter</code>. <br>
 * When the distant <code>Transporter</code> receives the
 * <code>MessageExchange</code>, it has to call <code>receive(exchange)</code>
 * on its local <code>Router</code>. <br>
 * The localization of the distant container is in charge of the
 * <code>Transporter</code>. <br>
 * <br>
 * <em>Note that the destination container can be the same than this one.</em>
 * 
 * @version $Rev: 477 $ $Date: 2006-05-29 17:18:07 +0200 (lun., 29 mai 2006) $
 * @since Petals 1.0
 * @author alouis - EBM Websourcing
 * @author rnaudin - EBM Websourcing
 */
public interface Transporter {

    /**
     * The list of provided transporter. Note that in standalone distribution,
     * only the <code>LOCAL_FRACTAL_TRANSPORTER</code> is supported
     */
    static final String LOCAL_FRACTAL_TRANSPORTER = "local";

    static final String TCP_FRACTAL_TRANSPORTER = "tcp";

    /*TODO Not yet implemented! */
    static final String SSL_FRACTAL_TRANSPORTER = "ssl";

    /*TODO Not yet implemented! */
    static final String HTTP_FRACTAL_TRANSPORTER = "http";

    /**
     * Send the specified exchange to the specified container. According to the
     * transport policy, the message is may be not persisted by the Transport
     * layer, so it can be lost in case of problem.
     * 
     * @param exchange
     *            the exchange to send. <b>MUST</b> be non null
     * @param componentName
     *            the name of the distant container. <b>MUST</b> be non null
     * @param containerName
     *            the name of the distant container. <b>MUST</b> be non null and
     *            non empty
     * @param subDomainName
     *            the name of the sub-domain of the remote container.
     *            <b>MUST</b> be non null and non empty
     * @param timeToLive
     *            duration of the message life (in ms). 0 for no timeout
     * @throws TransportException
     *             If the exchange is invalid, or can not be serialized. If the
     *             distant container can not be found.
     */
    void send(final MessageExchange exchange, final TransportSendContext transportContext)
            throws TransportException;

    /**
     * Send the specified exchange to the specified container and wait until the
     * response is received or the timeout is reached. According to the
     * transport policy, the message is may be not persisted by the Transport
     * layer, so it can be lost in case of problem.
     * 
     * @param exchange
     *            the exchange to send. <b>MUST</b> be non null
     * @param componentName
     *            the name of the distant container. <b>MUST</b> be non null
     * @param containerName
     *            the name of the distant container. <b>MUST</b> be non null and
     *            non empty
     * @param subDomainName
     *            the name of the sub-domain of the remote container.
     *            <b>MUST</b> be non null and non empty
     * @param timeToLive
     *            duration of the message life (in ms). 0 for no timeout
     * @throws TransportException
     *             If the exchange is invalid, or can not be serialized. If the
     *             distant container can not be found.
     */
    MessageExchange sendSync(final MessageExchange exchange,
            final TransportSendContext transportContext) throws TransportException;

    /**
     * Prepare the stopping of the component. It means that all ongoing message
     * traffic must terminate during the method invocation and all new message
     * traffic is rejected.
     */
    void stopTraffic();
}
