/**
 * PETALS: PETALS Services Platform Copyright (C) 2009 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 */
package org.ow2.petals.ws;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import javax.activation.DataHandler;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.configuration.ConfigurationService;
import org.ow2.petals.kernel.ws.api.ArtifactRepositoryService;
import org.ow2.petals.kernel.ws.api.PEtALSWebServiceException;
import org.ow2.petals.kernel.ws.api.to.AttachmentDescriptor;
import org.ow2.petals.system.repository.artifact.Artifact;
import org.ow2.petals.tools.ws.KernelWebService;
import org.ow2.petals.util.LoggingUtil;

/**
 * @author chamerling - eBM WebSourcing
 * 
 */
@FractalComponent
@Provides(interfaces = { @Interface(name = "service", signature = KernelWebService.class),
        @Interface(name = "webservice", signature = ArtifactRepositoryService.class) })
public class ArtifactRepositoryServiceImpl implements KernelWebService, ArtifactRepositoryService {

    @Monolog(name = "logger")
    private Logger logger;

    private LoggingUtil log;

    @org.objectweb.fractal.fraclet.annotation.annotations.Service(name = "component")
    private Component component;

    @Requires(name = "artifactrepository", signature = org.ow2.petals.system.repository.artifact.ArtifactRepositoryService.class)
    private org.ow2.petals.system.repository.artifact.ArtifactRepositoryService artifactRepositoryService;

    @Requires(name = "configuration", signature = ConfigurationService.class)
    private ConfigurationService configurationService;

    private File uploadPath;

    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(this.logger);
        this.log.debug("Starting...");

        this.uploadPath = new File(this.configurationService.getContainerConfiguration()
                .getWorkDirectoryPath(), "upload");
        if (!this.uploadPath.exists()) {
            this.uploadPath.mkdirs();
        }
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.debug("Stopping...");
    }

    /**
     * {@inheritDoc}
     */
    public Component getComponent() {
        return this.component;
    }

    public boolean addArtifact(AttachmentDescriptor descriptor) throws PEtALSWebServiceException {
        boolean result = false;
        File out = null;
        DataHandler dh = descriptor.getAttachment();
        if (dh != null) {
            try {
                out = new File(this.uploadPath, "artifactUpload-" + System.currentTimeMillis());
                dh.writeTo(new FileOutputStream(out));

                // try to add this as a repository artifact
                Artifact artifact = new Artifact();
                artifact.setFile(out);

                result = this.artifactRepositoryService.addArtifact(artifact);
                if (result) {
                    out.delete();
                } else {
                    // FIXME : Delete it or move it somewhere in the trash
                    // folder
                    throw new PEtALSWebServiceException(
                            "The file has not been added to the artifact repository service");
                }
            } catch (IOException e) {
                throw new PEtALSWebServiceException(e.getMessage());
            } catch (PetalsException e) {
                throw new PEtALSWebServiceException(e.getMessage());
            }
        }
        return result;
    }

    public String[] getComponents() {
        final List<String> components = this.artifactRepositoryService.getComponentsName();
        return components.toArray(new String[0]);
    }

    public String[] getServiceAssemblies() {
        final List<String> sas = this.artifactRepositoryService.getServiceAssembliesName();
        return sas.toArray(new String[0]);
    }

    public String[] getSharedLibraries() {
        final List<String> sls = this.artifactRepositoryService.getSharedLibrariesName();
        return sls.toArray(new String[0]);
    }

}
