/**
 * PETALS: PETALS Services Platform Copyright (C) 2009 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 */
package org.ow2.petals.ws;

import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.transform.TransformerException;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.jbi.messaging.endpoint.ServiceEndpoint;
import org.ow2.petals.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.jbi.messaging.registry.RegistryException;
import org.ow2.petals.kernel.ws.api.EndpointService;
import org.ow2.petals.kernel.ws.api.PEtALSWebServiceException;
import org.ow2.petals.kernel.ws.api.to.Endpoint;
import org.ow2.petals.kernel.ws.api.to.EndpointQuery;
import org.ow2.petals.tools.ws.KernelWebService;
import org.ow2.petals.util.LoggingUtil;
import org.ow2.petals.util.XMLUtil;

/**
 * @author chamerling - eBM WebSourcing
 * 
 */
@FractalComponent
@Provides(interfaces = { @Interface(name = "service", signature = KernelWebService.class),
        @Interface(name = "webservice", signature = EndpointService.class) })
public class EndpointServiceImpl implements KernelWebService, EndpointService {

    @org.objectweb.fractal.fraclet.annotation.annotations.Service(name = "component")
    private Component component;

    @Monolog(name = "logger")
    private Logger logger;

    private LoggingUtil log;

    @Requires(name = "endpoint", signature = EndpointRegistry.class)
    private EndpointRegistry endpointRegistry;

    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(this.logger);
        this.log.debug("Starting...");
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.debug("Stopping...");
    }

    /**
     * {@inheritDoc}
     */
    public Component getComponent() {
        return this.component;
    }

    /**
     * {@inheritDoc}
     */
    public List<Endpoint> getEndpoints() throws PEtALSWebServiceException {
        List<Endpoint> result = new ArrayList<Endpoint>();

        List<ServiceEndpoint> endpoints = null;
        try {
            endpoints = this.endpointRegistry.getEndpoints();
        } catch (RegistryException e) {
            final String message = "Can not get endpoint list";
            this.log.warning(message, e);
            throw new PEtALSWebServiceException(message);
        }

        for (ServiceEndpoint serviceEndpoint : endpoints) {
            Endpoint endpoint = new Endpoint();
            endpoint.setComponent(serviceEndpoint.getLocation().getComponentName());
            endpoint.setContainer(serviceEndpoint.getLocation().getContainerName());
            endpoint.setInterfaces(serviceEndpoint.getInterfaces());
            endpoint.setName(serviceEndpoint.getEndpointName());
            endpoint.setService(serviceEndpoint.getServiceName());
            endpoint.setSubDomain(serviceEndpoint.getLocation().getSubdomainName());

            try {
                endpoint.setDescription(XMLUtil.createStringFromDOMDocument(serviceEndpoint
                        .getDescription()));
            } catch (TransformerException e) {
                this.log.warning(e.getMessage());
            }

            result.add(endpoint);
        }

        return result;
    }

    /**
     * {@inheritDoc}
     */
    public List<Endpoint> query(EndpointQuery query) throws PEtALSWebServiceException {
        if (query == null) {
            throw new PEtALSWebServiceException("Query can not be null");
        }
        List<Endpoint> result = new ArrayList<Endpoint>();
        List<ServiceEndpoint> endpoints = null;
        String endpointName = null;
        if (query.getEndpoint() != null) {
            endpointName = query.getEndpoint().toString();
        }
        try {
            endpoints = this.endpointRegistry.query(endpointName, query.getInterface(), query
                    .getService(), query.getContainerName(), query.getComponentName(), query
                    .getSubdomainName(), query.getType());
        } catch (RegistryException e) {
            final String message = "Can not get endpoint list";
            this.log.warning(message, e);
            throw new PEtALSWebServiceException(message);
        }

        for (ServiceEndpoint serviceEndpoint : endpoints) {

            Endpoint endpoint = new Endpoint();
            endpoint.setComponent(serviceEndpoint.getLocation().getComponentName());
            endpoint.setContainer(serviceEndpoint.getLocation().getContainerName());
            final List<QName> interfaceNames = serviceEndpoint.getInterfacesName();
            if (interfaceNames != null) {
                endpoint.setInterfaces(interfaceNames.toArray(new QName[interfaceNames.size()]));
            }
            endpoint.setName(serviceEndpoint.getEndpointName());
            endpoint.setService(serviceEndpoint.getServiceName());
            endpoint.setSubDomain(serviceEndpoint.getLocation().getSubdomainName());
            try {
                endpoint.setDescription(XMLUtil.createStringFromDOMDocument(serviceEndpoint
                        .getDescription()));
            } catch (TransformerException e) {
                this.log.warning(e.getMessage());
            }
            result.add(endpoint);
        }
        return result;
    }

}
